import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/widget.dart';

/// Text field supports name format.
class NameFieldWidget extends OneValueFieldWidget {
  NameFieldWidget(
      {final TextInputAction action = TextInputAction.next,
      final BoxDecoration? decoration,
      final bool enabled = true,
      final BoxDecoration? errorDecoration,
      final TextStyle? errorStyle,
      final String? hint,
      final String? initialValue,
      final Key? key,
      final Function(dynamic value)? onChanged,
      final TextStyle? style})
      : super(
            action: action,
            decoration: decoration,
            enabled: enabled,
            errorDecoration: errorDecoration,
            errorStyle: errorStyle,
            hint: hint,
            initialValue: initialValue,
            key: key,
            onChanged: onChanged,
            style: style);

  @override
  State<StatefulWidget> createState() => _State();
}

class _State extends State<NameFieldWidget>
    with FieldMixin, OneValueFieldMixin {
  @override
  Widget build(final BuildContext context) => container(
      decoration: valid ? widget.decoration : widget.errorDecoration,
      textField: TextField(
          controller: controller,
          decoration:
              InputDecoration(border: InputBorder.none, hintText: widget.hint),
          enabled: widget.enabled,
          inputFormatters: [
            FilteringTextInputFormatter.allow(RegExp('[a-z A-Z]'))
          ],
          keyboardType: TextInputType.name,
          style: valid ? widget.style : widget.errorStyle,
          textCapitalization: TextCapitalization.words,
          textInputAction: widget.action));

  @override
  void dispose() {
    disposeMixin();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    initStateMixin(
        initialValue: widget.initialValue, onChanged: widget.onChanged);
  }

  @override
  String get value => super.value;
}
