import 'package:flutter/material.dart';
import 'package:flutter_custom_form_component/widget/color.dart';

abstract class FieldWidget extends StatefulWidget {
  late final TextInputAction _action;
  late final bool _enabled;
  late final BoxDecoration _decoration, _errorDecoration;
  late final TextStyle _errorStyle, _style;
  late final String? _hint;

  FieldWidget(
      {final TextInputAction action = TextInputAction.next,
      final BoxDecoration? decoration,
      final bool enabled = true,
      final BoxDecoration? errorDecoration,
      final TextStyle? errorStyle,
      final String? hint,
      final Key? key,
      final TextStyle? style})
      : super(key: key) {
    _action = action;
    _decoration = decoration ?? _boxDecoration();
    _enabled = enabled;
    _errorDecoration = errorDecoration ?? _boxDecoration(forError: true);
    _errorStyle = errorStyle ?? _textStyle(forError: true);
    _hint = hint;
    _style = style ?? _textStyle();
  }

  BoxDecoration _boxDecoration({final bool forError = false}) => BoxDecoration(
      border: Border.all(color: forError ? colorRed : colorBlack, width: 0.5),
      borderRadius: BorderRadius.circular(8.0),
      color: colorWhite);

  TextStyle _textStyle({final bool forError = false}) => TextStyle(
      color: forError ? colorRed : colorBlack,
      fontFamily: 'Circular Std',
      fontSize: 16.0,
      fontWeight: FontWeight.w400);

  TextInputAction get action => _action;

  BoxDecoration get decoration => _decoration;

  bool get enabled => _enabled;

  BoxDecoration get errorDecoration => _errorDecoration;

  TextStyle get errorStyle => _errorStyle;

  String? get hint => _hint;

  TextStyle get style => _style;
}
